@@ -31,7 +31,7 @@ class CControllerMediatypeEdit extends CController {
 		$fields = [
 			'mediatypeid' =>			'db media_type.mediatypeid',
 			'type' =>					'db media_type.type|in '.implode(',', array_keys(media_type2str())),
-			'description' =>			'db media_type.description',
+			'name' =>					'db media_type.name',
 			'smtp_server' =>			'db media_type.smtp_server',
 			'smtp_port' =>				'db media_type.smtp_port',
 			'smtp_helo' =>				'db media_type.smtp_helo',
@@ -41,17 +41,22 @@ class CControllerMediatypeEdit extends CController {
 			'smtp_verify_host' =>		'db media_type.smtp_verify_host|in 0,1',
 			'smtp_authentication' =>	'db media_type.smtp_authentication|in '.SMTP_AUTHENTICATION_NONE.','.SMTP_AUTHENTICATION_NORMAL,
 			'exec_path' =>				'db media_type.exec_path',
-			'eztext_limit' =>			'in '.EZ_TEXTING_LIMIT_USA.','.EZ_TEXTING_LIMIT_CANADA,
 			'exec_params' =>			'array',
 			'gsm_modem' =>				'db media_type.gsm_modem',
-			'jabber_username' =>		'db media_type.username',
-			'eztext_username' =>		'db media_type.username',
 			'smtp_username' =>			'db media_type.username',
 			'passwd' =>					'db media_type.passwd',
+			'parameters' =>				'array',
+			'script' => 				'db media_type.script',
+			'timeout' => 				'db media_type.timeout',
+			'process_tags' =>			'in '.ZBX_MEDIA_TYPE_TAGS_DISABLED.','.ZBX_MEDIA_TYPE_TAGS_ENABLED,
+			'show_event_menu' =>		'in '.ZBX_EVENT_MENU_HIDE.','.ZBX_EVENT_MENU_SHOW,
+			'event_menu_url' =>			'db media_type.event_menu_url',
+			'event_menu_name' =>		'db media_type.event_menu_name',
 			'status' =>					'db media_type.status|in '.MEDIA_TYPE_STATUS_ACTIVE.','.MEDIA_TYPE_STATUS_DISABLED,
 			'maxsessions' =>			'db media_type.maxsessions',
 			'maxattempts' =>			'db media_type.maxattempts',
 			'attempt_interval' =>		'db media_type.attempt_interval',
+			'description' =>			'db media_type.description',
 			'form_refresh' =>			'int32',
 			'content_type' =>			'db media_type.content_type|in '.SMTP_MESSAGE_FORMAT_PLAIN_TEXT.','.SMTP_MESSAGE_FORMAT_HTML
 		];
@@ -82,10 +87,11 @@ class CControllerMediatypeEdit extends CController {
 
 		if ($this->hasInput('mediatypeid')) {
 			$mediatypes = API::Mediatype()->get([
-				'output' => ['mediatypeid', 'type', 'description', 'smtp_server', 'smtp_port', 'smtp_helo',
-					'smtp_email', 'exec_path', 'gsm_modem', 'username', 'passwd', 'status', 'smtp_security',
-					'smtp_verify_peer', 'smtp_verify_host', 'smtp_authentication', 'exec_params', 'maxsessions',
-					'maxattempts', 'attempt_interval', 'content_type'
+				'output' => ['mediatypeid', 'type', 'name', 'smtp_server', 'smtp_port', 'smtp_helo', 'smtp_email',
+					'exec_path', 'gsm_modem', 'username', 'passwd', 'status', 'smtp_security', 'smtp_verify_peer',
+					'smtp_verify_host', 'smtp_authentication', 'exec_params', 'maxsessions', 'maxattempts',
+					'attempt_interval', 'content_type', 'script', 'timeout', 'process_tags', 'show_event_menu',
+					'event_menu_url', 'event_menu_name', 'parameters', 'description'
 				],
 				'mediatypeids' => $this->getInput('mediatypeid'),
 				'editable' => true
@@ -108,7 +114,7 @@ class CControllerMediatypeEdit extends CController {
 			'sid' => $this->getUserSID(),
 			'mediatypeid' => 0,
 			'type' => MEDIA_TYPE_EMAIL,
-			'description' => '',
+			'name' => '',
 			'smtp_server' => 'mail.example.com',
 			'smtp_port' => $db_defaults['smtp_port'],
 			'smtp_helo' => 'example.com',
@@ -120,9 +126,6 @@ class CControllerMediatypeEdit extends CController {
 			'exec_params' => [],
 			'exec_path' => '',
 			'gsm_modem' => '/dev/ttyS0',
-			'jabber_username' => 'user@server',
-			'eztext_username' => '',
-			'eztext_limit' => EZ_TEXTING_LIMIT_USA,
 			'smtp_username' => '',
 			'passwd' => '',
 			'status' => MEDIA_TYPE_STATUS_ACTIVE,
@@ -130,6 +133,19 @@ class CControllerMediatypeEdit extends CController {
 			'maxsessions' => $db_defaults['maxsessions'],
 			'maxattempts' => $db_defaults['maxattempts'],
 			'attempt_interval' => $db_defaults['attempt_interval'],
+			'script' => $db_defaults['script'],
+			'timeout' => $db_defaults['timeout'],
+			'process_tags' => $db_defaults['process_tags'],
+			'show_event_menu' => $db_defaults['show_event_menu'],
+			'event_menu_url' => $db_defaults['event_menu_url'],
+			'event_menu_name' => $db_defaults['event_menu_name'],
+			'parameters' => [
+				['name' => 'URL', 'value'=> ''],
+				['name' => 'To', 'value' => '{ALERT.SENDTO}'],
+				['name' => 'Subject', 'value' => '{ALERT.SUBJECT}'],
+				['name' => 'Message', 'value' => '{ALERT.MESSAGE}']
+			],
+			'description' => '',
 			'form_refresh' => 0,
 			'content_type' => $db_defaults['content_type']
 		];
@@ -138,7 +154,7 @@ class CControllerMediatypeEdit extends CController {
 		if ($this->hasInput('mediatypeid')) {
 			$data['mediatypeid'] = $this->mediatype['mediatypeid'];
 			$data['type'] = $this->mediatype['type'];
-			$data['description'] = $this->mediatype['description'];
+			$data['name'] = $this->mediatype['name'];
 			$data['smtp_server'] = $this->mediatype['smtp_server'];
 			$data['smtp_port'] = $this->mediatype['smtp_port'];
 			$data['smtp_helo'] = $this->mediatype['smtp_helo'];
@@ -149,6 +165,7 @@ class CControllerMediatypeEdit extends CController {
 			$data['smtp_authentication'] = $this->mediatype['smtp_authentication'];
 			$data['exec_path'] = $this->mediatype['exec_path'];
 			$data['content_type'] = $this->mediatype['content_type'];
+			$data['description'] = $this->mediatype['description'];
 
 			$this->mediatype['exec_params'] = explode("\n", $this->mediatype['exec_params']);
 			foreach ($this->mediatype['exec_params'] as $exec_param) {
@@ -169,52 +186,45 @@ class CControllerMediatypeEdit extends CController {
 					$data['smtp_username'] = $this->mediatype['username'];
 					break;
 
-				case MEDIA_TYPE_JABBER:
-					$data['jabber_username'] = $this->mediatype['username'];
-					break;
-
-				case MEDIA_TYPE_EZ_TEXTING:
-					$data['eztext_username'] = $this->mediatype['username'];
-					$data['eztext_limit'] = $this->mediatype['exec_path'];
-					break;
-
 				case MEDIA_TYPE_SMS:
 					$data['maxsessions'] = 1;
 					break;
+
+				case MEDIA_TYPE_WEBHOOK:
+					$data['script'] = $this->mediatype['script'];
+					$data['timeout'] = $this->mediatype['timeout'];
+					$data['process_tags'] = $this->mediatype['process_tags'];
+					$data['show_event_menu'] = $this->mediatype['show_event_menu'];
+					$data['event_menu_url'] = $this->mediatype['event_menu_url'];
+					$data['event_menu_name'] = $this->mediatype['event_menu_name'];
+					$data['parameters'] = $this->mediatype['parameters'];
+					break;
 			}
 
 			$data['change_passwd'] = $this->hasInput('passwd');
 		}
 
 		// overwrite with input variables
-		$this->getInputs($data, [
-			'type',
-			'description',
-			'smtp_server',
-			'smtp_port',
-			'smtp_helo',
-			'smtp_email',
-			'smtp_security',
-			'smtp_verify_peer',
-			'smtp_verify_host',
-			'smtp_authentication',
-			'exec_params',
-			'exec_path',
-			'eztext_limit',
-			'gsm_modem',
-			'jabber_username',
-			'eztext_username',
-			'smtp_username',
-			'passwd',
-			'status',
-			'maxsessions',
-			'maxattempts',
-			'attempt_interval',
-			'maxsessionsType',
-			'form_refresh',
-			'content_type'
+		$this->getInputs($data, ['type', 'name', 'smtp_server', 'smtp_port', 'smtp_helo', 'smtp_email', 'smtp_security',
+			'smtp_verify_peer', 'smtp_verify_host', 'smtp_authentication', 'exec_params', 'exec_path', 'gsm_modem',
+			'smtp_username', 'passwd', 'status', 'maxsessions', 'maxattempts', 'attempt_interval', 'maxsessionsType',
+			'form_refresh', 'content_type', 'script', 'timeout', 'process_tags', 'show_event_menu', 'event_menu_url',
+			'event_menu_name', 'description'
 		]);
 
+		if ($this->hasInput('form_refresh')) {
+			$data['parameters'] = [];
+			$parameters = $this->getInput('parameters', ['name' => [], 'value' => []]);
+			$name = reset($parameters['name']);
+			$value = reset($parameters['value']);
+
+			while ($name !== false) {
+				$data['parameters'][] = compact('name', 'value');
+				$name = next($parameters['name']);
+				$value = next($parameters['value']);
+			}
+		}
+
 		$response = new CControllerResponseData($data);
 		$response->setTitle(_('Configuration of media types'));
 		$this->setResponse($response);
