@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@ else {
 	$this->addJsFile('csvggraphwidget.js');
 	$this->addJsFile('class.cclock.js');
 	$this->addJsFile('class.cnavtree.js');
-	$this->addJsFile('class.mapWidget.js');
+	$this->addJsFile('class.mapwidget.js');
 	$this->addJsFile('class.svg.canvas.js');
 	$this->addJsFile('class.svg.map.js');
 
@@ -66,20 +66,6 @@ else {
 		);
 	}
 
-	$url_create = (new CUrl('zabbix.php'))
-		->setArgument('action', 'dashboard.view')
-		->setArgument('new', '1');
-	$url_clone = (new CUrl('zabbix.php'))
-		->setArgument('action', 'dashboard.view')
-		->setArgument('source_dashboardid', $data['dashboard']['dashboardid']);
-
-	if ($data['dashboard']['editable']) {
-		$url_delete = (new CUrl('zabbix.php'))
-			->setArgument('action', 'dashboard.delete')
-			->setArgument('dashboardids', [$data['dashboard']['dashboardid']])
-			->setArgumentSID();
-	}
-
 	$web_layout_mode = CView::getLayoutMode();
 
 	$widget = (new CWidget())
@@ -90,48 +76,18 @@ else {
 			->addItem($main_filter_form)
 			->addItem((new CTag('nav', true, [
 				(new CList())
-					->addItem((
-						(new CButton('dashbrd-edit', _('Edit dashboard')))
-							->setEnabled($data['dashboard']['editable'])
-							->setAttribute('aria-disabled', !$data['dashboard']['editable'] ? 'true' : null)
-					))
+					->addItem((new CButton('dashbrd-edit', _('Edit dashboard')))
+						->setEnabled($data['dashboard']['editable'])
+						->setAttribute('aria-disabled', !$data['dashboard']['editable'] ? 'true' : null)
+					)
 					->addItem((new CButton('', '&nbsp;'))
 						->addClass(ZBX_STYLE_BTN_ACTION)
 						->setId('dashbrd-actions')
 						->setTitle(_('Actions'))
 						->setAttribute('aria-haspopup', true)
-						->setMenuPopup([
-							'type' => 'dashboard',
-							'label' => _('Actions'),
-							'items' => [
-								'sharing' => [
-									'label' => _('Sharing'),
-									'form_data' => [
-										'dashboardid' => $data['dashboard']['dashboardid']
-									],
-									'disabled' => !$data['dashboard']['editable']
-								],
-							'create' => [
-								'label' => _('Create new'),
-								'url' => $url_create->getUrl()
-							],
-							'clone' => [
-								'label' => _('Clone'),
-								'url' => $url_clone->getUrl()
-							],
-							'delete' => [
-								'label' => _('Delete'),
-								'confirmation' => _('Delete dashboard?'),
-								'url' => 'javascript:void(0)',
-								'redirect' => $data['dashboard']['editable']
-									? $url_delete->getUrl()
-									: null,
-								'disabled' => !$data['dashboard']['editable']
-							]
-						]
-					])
-				)
-				->addItem(get_icon('fullscreen'))
+						->setMenuPopup(CMenuPopupHelper::getDashboard($data['dashboard']['dashboardid']))
+					)
+					->addItem(get_icon('fullscreen'))
 			]))->setAttribute('aria-label', _('Content controls'))
 		)
 		->addItem((new CListItem([
@@ -169,7 +125,7 @@ else {
 
 	$widget
 		->addItem($timeline)
-		->addItem((new CDiv())->addClass(ZBX_STYLE_DASHBRD_GRID_WIDGET_CONTAINER))
+		->addItem((new CDiv())->addClass(ZBX_STYLE_DASHBRD_GRID_CONTAINER))
 		->show();
 
 	/*
@@ -192,8 +148,11 @@ else {
 	$dashboard_options = [
 		'max-rows' => DASHBOARD_MAX_ROWS,
 		'max-columns' => DASHBOARD_MAX_COLUMNS,
+		'widget-min-rows' => DASHBOARD_WIDGET_MIN_ROWS,
+		'widget-max-rows' => DASHBOARD_WIDGET_MAX_ROWS,
 		'editable' => $data['dashboard']['editable'],
-		'edit_mode' => $data['dashboard_edit_mode']
+		'edit_mode' => $data['dashboard_edit_mode'],
+		'kioskmode' => ($web_layout_mode === ZBX_LAYOUT_KIOSKMODE)
 	];
 	if ($data['dashboard']['dashboardid'] != 0) {
 		$dashboard_data['id'] = $data['dashboard']['dashboardid'];
@@ -205,7 +164,6 @@ else {
 	// must be done before adding widgets, because it causes dashboard to resize.
 	if ($data['show_timeselector']) {
 		$this->addPostJS(
-			'timeControl.useTimeRefresh('.CWebUser::getRefresh().');'.
 			'timeControl.addObject("scrollbar", '.CJs::encodeJson($data['timeline']).', '.
 				CJs::encodeJson($data['timeControlData']).
 			');'.
@@ -213,13 +171,9 @@ else {
 		);
 	}
 
-	if ($web_layout_mode === ZBX_LAYOUT_KIOSKMODE) {
-		$dashboard_options['kioskmode'] = true;
-	}
-
 	// Initialize dashboard grid.
 	$this->addPostJS(
-		'jQuery(".'.ZBX_STYLE_DASHBRD_GRID_WIDGET_CONTAINER.'")'.
+		'jQuery(".'.ZBX_STYLE_DASHBRD_GRID_CONTAINER.'")'.
 			'.dashboardGrid('.CJs::encodeJson($dashboard_options).')'.
 			'.dashboardGrid("setDashboardData", '.CJs::encodeJson($dashboard_data).')'.
 			'.dashboardGrid("setWidgetDefaults", '.CJs::encodeJson($data['widget_defaults']).')'.
