@@ -54,11 +54,13 @@
 
 		$('#filter_compact_view').change(function() {
 			if ($(this).is(':checked')) {
-				$('#filter_show_timeline, #filter_details, #filter_show_latest_values').prop('disabled', true);
+				$('#filter_show_timeline, #filter_details').prop('disabled', true);
+				$('input[name=filter_show_opdata]').prop('disabled', true);
 				$('#filter_highlight_row').prop('disabled', false);
 			}
 			else {
-				$('#filter_show_timeline, #filter_details, #filter_show_latest_values').prop('disabled', false);
+				$('#filter_show_timeline, #filter_details').prop('disabled', false);
+				$('input[name=filter_show_opdata]').prop('disabled', false);
 				$('#filter_highlight_row').prop('disabled', true);
 			}
 		});
@@ -71,7 +73,7 @@
 
 		$(document).on({
 			mouseenter: function() {
-				if ($(this).width() > $(this).parent('td').width()) {
+				if ($(this)[0].scrollWidth > $(this)[0].offsetWidth) {
 					$(this).attr({title: $(this).text()});
 				}
 			},
