@@ -48,10 +48,7 @@ if (isset($_REQUEST['width']) || isset($_REQUEST['height'])) {
 }
 
 if (isset($_REQUEST['css'])) {
-	$css = 'div.sysmap_iconid_0 {'.
-			' height: 50px;'.
-			' width: 50px;'.
-			' background-image: url("images/general/no_icon.png"); }'."\n";
+	$css = '';
 
 	$images = API::Image()->get([
 		'output' => ['imageid'],
