@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -23,6 +23,9 @@ class CPieGraphDraw extends CGraphDraw {
 
 	const DEFAULT_HEADER_PADDING_TOP = 30;
 
+	const GRAPH_WIDTH_MIN = 20;
+	const GRAPH_HEIGHT_MIN = 20;
+
 	public function __construct($type = GRAPH_TYPE_PIE) {
 		parent::__construct($type);
 		$this->background = false;
@@ -139,7 +142,7 @@ class CPieGraphDraw extends CGraphDraw {
 
 	protected function selectData() {
 		$this->data = [];
-		$now = time(null);
+		$now = time();
 
 		if (isset($this->stime)) {
 			$this->from_time = $this->stime;
@@ -189,7 +192,7 @@ class CPieGraphDraw extends CGraphDraw {
 				$to_resolve[] = 'trends';
 			}
 
-			// Otherwise, resolve user macro and parse the string. If successfull, convert to seconds.
+			// Otherwise, resolve user macro and parse the string. If successful, convert to seconds.
 			if ($to_resolve) {
 				$item = CMacrosResolverHelper::resolveTimeUnitMacros([$item], $to_resolve)[0];
 
@@ -228,7 +231,7 @@ class CPieGraphDraw extends CGraphDraw {
 			$items[] = $item;
 		}
 
-		$results = Manager::History()->getGraphAggregation($items, $from_time, $to_time);
+		$results = Manager::History()->getGraphAggregationByWidth($items, $from_time, $to_time);
 		$i = 0;
 
 		foreach ($items as $item) {
@@ -473,9 +476,15 @@ class CPieGraphDraw extends CGraphDraw {
 
 		$anglestart = 0;
 		$angleend = 0;
+
 		foreach ($values as $item => $value) {
+			if ($value == 0) {
+				continue;
+			}
+
 			$angleend += (int) (360 * $value / $sum) + 1;
 			$angleend = ($angleend > 360) ? 360 : $angleend;
+
 			if (($angleend - $anglestart) < 1) {
 				continue;
 			}
@@ -549,9 +558,15 @@ class CPieGraphDraw extends CGraphDraw {
 		// bottom angle line
 		$anglestart = 0;
 		$angleend = 0;
+
 		foreach ($values as $item => $value) {
+			if ($value == 0) {
+				continue;
+			}
+
 			$angleend += (int) (360 * $value / $sum) + 1;
 			$angleend = ($angleend > 360) ? 360 : $angleend;
+
 			if (($angleend - $anglestart) < 1) {
 				continue;
 			}
@@ -559,6 +574,7 @@ class CPieGraphDraw extends CGraphDraw {
 			if ($this->type == GRAPH_TYPE_3D_EXPLODED) {
 				list($x, $y) = $this->calcExplodedCenter($anglestart, $angleend, $xc, $yc, count($values));
 			}
+
 			imagefilledarc(
 				$this->im,
 				$x,
@@ -588,7 +604,12 @@ class CPieGraphDraw extends CGraphDraw {
 		for ($i = $this->graphheight3d; $i > 0; $i--) {
 			$anglestart = 0;
 			$angleend = 0;
+
 			foreach ($values as $item => $value) {
+				if ($value == 0) {
+					continue;
+				}
+
 				$angleend += (int) (360 * $value / $sum) + 1;
 				$angleend = ($angleend > 360) ? 360 : $angleend;
 
@@ -620,9 +641,15 @@ class CPieGraphDraw extends CGraphDraw {
 
 		$anglestart = 0;
 		$angleend = 0;
+
 		foreach ($values as $item => $value) {
+			if ($value == 0) {
+				continue;
+			}
+
 			$angleend += (int) (360 * $value / $sum) + 1;
 			$angleend = ($angleend > 360) ? 360 : $angleend;
+
 			if (($angleend - $anglestart) < 1) {
 				continue;
 			}
