@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -106,7 +106,7 @@ class CMacrosResolverGeneral {
 	 * @return array
 	 */
 	protected function transformToPositionTypes(array $types) {
-		foreach (['macros', 'macros_n', 'macro_funcs_n'] as $type) {
+		foreach (['macros', 'macros_n', 'macros_an', 'macro_funcs_n'] as $type) {
 			if (array_key_exists($type, $types)) {
 				$patterns = [];
 				foreach ($types[$type] as $key => $_patterns) {
@@ -127,6 +127,7 @@ class CMacrosResolverGeneral {
 	 * @param bool   $types['usermacros']
 	 * @param array  $types['macros'][<macro_patterns>]
 	 * @param array  $types['macros_n'][<macro_patterns>]
+	 * @param array  $types['macros_an'][<macro_patterns>]
 	 * @param array  $types['macro_funcs_n'][<macro_patterns>]
 	 * @param bool   $types['references']
 	 * @param bool   $types['lldmacros']
@@ -135,11 +136,12 @@ class CMacrosResolverGeneral {
 	 *
 	 * @return array
 	 */
-	protected function getMacroPositions($text, array $types) {
+	public function getMacroPositions($text, array $types) {
 		$macros = [];
 		$extract_usermacros = array_key_exists('usermacros', $types);
 		$extract_macros = array_key_exists('macros', $types);
 		$extract_macros_n = array_key_exists('macros_n', $types);
+		$extract_macros_an = array_key_exists('macros_an', $types);
 		$extract_macro_funcs_n = array_key_exists('macro_funcs_n', $types);
 		$extract_references = array_key_exists('references', $types);
 		$extract_lldmacros = array_key_exists('lldmacros', $types);
@@ -155,11 +157,19 @@ class CMacrosResolverGeneral {
 		}
 
 		if ($extract_macros_n) {
-			$macro_n_parser = new CMacroParser($types['macros_n'], ['allow_reference' => true]);
+			$macro_n_parser = new CMacroParser($types['macros_n'], ['ref_type' => CMacroParser::REFERENCE_NUMERIC]);
+		}
+
+		if ($extract_macros_an) {
+			$macro_an_parser = new CMacroParser($types['macros_an'],
+				['ref_type' => CMacroParser::REFERENCE_ALPHANUMERIC]
+			);
 		}
 
 		if ($extract_macro_funcs_n) {
-			$macro_func_n_parser = new CMacroFunctionParser($types['macro_funcs_n'], ['allow_reference' => true]);
+			$macro_func_n_parser = new CMacroFunctionParser($types['macro_funcs_n'],
+				['ref_type' => CMacroParser::REFERENCE_NUMERIC]
+			);
 		}
 
 		if ($extract_references) {
@@ -168,6 +178,7 @@ class CMacrosResolverGeneral {
 
 		if ($extract_lldmacros) {
 			$lld_macro_parser = new CLLDMacroParser();
+			$lld_macro_function_parser = new CLLDMacroFunctionParser();
 		}
 
 		if ($extract_functionids) {
@@ -191,6 +202,10 @@ class CMacrosResolverGeneral {
 				$macros[$pos] = $macro_n_parser->getMatch();
 				$pos += $macro_n_parser->getLength() - 1;
 			}
+			elseif ($extract_macros_an && $macro_an_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
+				$macros[$pos] = $macro_an_parser->getMatch();
+				$pos += $macro_an_parser->getLength() - 1;
+			}
 			elseif ($extract_macro_funcs_n && $macro_func_n_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
 				$macros[$pos] = $macro_func_n_parser->getMatch();
 				$pos += $macro_func_n_parser->getLength() - 1;
@@ -203,6 +218,10 @@ class CMacrosResolverGeneral {
 				$macros[$pos] = $lld_macro_parser->getMatch();
 				$pos += $lld_macro_parser->getLength() - 1;
 			}
+			elseif ($extract_lldmacros && $lld_macro_function_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
+				$macros[$pos] = $lld_macro_function_parser->getMatch();
+				$pos += $lld_macro_function_parser->getLength() - 1;
+			}
 			elseif ($extract_functionids && $functionid_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
 				$macros[$pos] = $functionid_parser->getMatch();
 				$pos += $functionid_parser->getLength() - 1;
@@ -224,6 +243,7 @@ class CMacrosResolverGeneral {
 	 * @param bool   $types['usermacros']
 	 * @param array  $types['macros'][][<macro_patterns>]
 	 * @param array  $types['macros_n'][][<macro_patterns>]
+	 * @param array  $types['macros_an'][][<macro_patterns>]
 	 * @param array  $types['macro_funcs_n'][][<macro_patterns>]
 	 * @param bool   $types['references']
 	 * @param bool   $types['lldmacros']
@@ -231,11 +251,12 @@ class CMacrosResolverGeneral {
 	 *
 	 * @return array
 	 */
-	protected function extractMacros(array $texts, array $types) {
+	public function extractMacros(array $texts, array $types) {
 		$macros = [];
 		$extract_usermacros = array_key_exists('usermacros', $types);
 		$extract_macros = array_key_exists('macros', $types);
 		$extract_macros_n = array_key_exists('macros_n', $types);
+		$extract_macros_an = array_key_exists('macros_an', $types);
 		$extract_macro_funcs_n = array_key_exists('macro_funcs_n', $types);
 		$extract_references = array_key_exists('references', $types);
 		$extract_lldmacros = array_key_exists('lldmacros', $types);
@@ -260,16 +281,31 @@ class CMacrosResolverGeneral {
 			$macros['macros_n'] = [];
 
 			foreach ($types['macros_n'] as $key => $macro_patterns) {
-				$types['macros_n'][$key] = new CMacroParser($macro_patterns, ['allow_reference' => true]);
+				$types['macros_n'][$key] = new CMacroParser($macro_patterns,
+					['ref_type' => CMacroParser::REFERENCE_NUMERIC]
+				);
 				$macros['macros_n'][$key] = [];
 			}
 		}
 
+		if ($extract_macros_an) {
+			$macros['macros_an'] = [];
+
+			foreach ($types['macros_an'] as $key => $macro_patterns) {
+				$types['macros_an'][$key] = new CMacroParser($macro_patterns,
+					['ref_type' => CMacroParser::REFERENCE_ALPHANUMERIC]
+				);
+				$macros['macros_an'][$key] = [];
+			}
+		}
+
 		if ($extract_macro_funcs_n) {
 			$macros['macro_funcs_n'] = [];
 
 			foreach ($types['macro_funcs_n'] as $key => $macro_patterns) {
-				$types['macro_funcs_n'][$key] = new CMacroFunctionParser($macro_patterns, ['allow_reference' => true]);
+				$types['macro_funcs_n'][$key] = new CMacroFunctionParser($macro_patterns,
+					['ref_type' => CMacroParser::REFERENCE_NUMERIC]
+				);
 				$macros['macro_funcs_n'][$key] = [];
 			}
 		}
@@ -284,6 +320,7 @@ class CMacrosResolverGeneral {
 			$macros['lldmacros'] = [];
 
 			$lld_macro_parser = new CLLDMacroParser();
+			$lld_macro_function_parser = new CLLDMacroFunctionParser();
 		}
 
 		if ($extract_functionids) {
@@ -315,7 +352,7 @@ class CMacrosResolverGeneral {
 						if ($macro_n_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
 							$macros['macros_n'][$key][$macro_n_parser->getMatch()] = [
 								'macro' => $macro_n_parser->getMacro(),
-								'f_num' => $macro_n_parser->getN()
+								'f_num' => $macro_n_parser->getReference()
 							];
 							$pos += $macro_n_parser->getLength() - 1;
 							continue 2;
@@ -323,6 +360,19 @@ class CMacrosResolverGeneral {
 					}
 				}
 
+				if ($extract_macros_an) {
+					foreach ($types['macros_an'] as $key => $macro_an_parser) {
+						if ($macro_an_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
+							$macros['macros_an'][$key][$macro_an_parser->getMatch()] = [
+								'macro' => $macro_an_parser->getMacro(),
+								'f_num' => $macro_an_parser->getReference()
+							];
+							$pos += $macro_an_parser->getLength() - 1;
+							continue 2;
+						}
+					}
+				}
+
 				if ($extract_macro_funcs_n) {
 					foreach ($types['macro_funcs_n'] as $key => $macro_func_n_parser) {
 						if ($macro_func_n_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
@@ -344,7 +394,7 @@ class CMacrosResolverGeneral {
 
 							$macros['macro_funcs_n'][$key][$macro_func_n_parser->getMatch()] = [
 								'macro' => $macro_n_parser->getMacro(),
-								'f_num' => $macro_n_parser->getN(),
+								'f_num' => $macro_n_parser->getReference(),
 								'function' => $function_parser->getFunction(),
 								'parameters' => $function_parameters
 							];
@@ -360,10 +410,17 @@ class CMacrosResolverGeneral {
 					continue;
 				}
 
-				if ($extract_lldmacros && $lld_macro_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
-					$macros['lldmacros'][$lld_macro_parser->getMatch()] = null;
-					$pos += $lld_macro_parser->getLength() - 1;
-					continue;
+				if ($extract_lldmacros) {
+					if ($lld_macro_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
+						$macros['lldmacros'][$lld_macro_parser->getMatch()] = null;
+						$pos += $lld_macro_parser->getLength() - 1;
+						continue;
+					}
+					elseif ($lld_macro_function_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
+						$macros['lldmacros'][$lld_macro_function_parser->getMatch()] = null;
+						$pos += $lld_macro_function_parser->getLength() - 1;
+						continue;
+					}
 				}
 
 				if ($extract_functionids && $functionid_parser->parse($text, $pos) != CParser::PARSE_FAIL) {
@@ -672,34 +729,55 @@ class CMacrosResolverGeneral {
 	 *
 	 * @param array $macros
 	 * @param array $macros[<functionid>]
-	 * @param array $macros[<functionid>][<macro>]  an array of the tokens
-	 * @param array $triggers
+	 * @param array $macros[<functionid>][<macro>]  An array of the tokens.
 	 * @param array $macro_values
-	 * @param bool  $events			resolve {ITEM.VALUE} macro using 'clock' and 'ns' fields
+	 * @param array $triggers
+	 * @param array $options
+	 * @param bool  $options['events]               Resolve {ITEM.VALUE} macro using 'clock' and 'ns' fields.
+	 * @param bool  $options['html]
 	 *
 	 * @return array
 	 */
-	protected function getItemMacros(array $macros, array $triggers, array $macro_values, $events) {
+	protected function getItemMacros(array $macros, array $macro_values, array $triggers = [], array $options = []) {
 		if (!$macros) {
 			return $macro_values;
 		}
 
-		$functions = DbFetchArray(DBselect(
-			'SELECT f.triggerid,f.functionid,i.itemid,i.value_type,i.units,i.valuemapid'.
+		$options += [
+			'events' => false,
+			'html' => false
+		];
+
+		$functions = DBfetchArray(DBselect(
+			'SELECT f.triggerid,f.functionid,i.itemid,i.hostid,i.name,i.key_,i.value_type,i.units,i.valuemapid'.
 			' FROM functions f'.
 				' JOIN items i ON f.itemid=i.itemid'.
 				' JOIN hosts h ON i.hostid=h.hostid'.
 			' WHERE '.dbConditionInt('f.functionid', array_keys($macros))
 		));
 
+		$functions = CMacrosResolverHelper::resolveItemNames($functions);
+
 		// False passed to DBfetch to get data without null converted to 0, which is done by default.
 		foreach ($functions as $function) {
-			foreach ($macros[$function['functionid']] as $macro => $tokens) {
-				switch ($macro) {
+			foreach ($macros[$function['functionid']] as $m => $tokens) {
+				$clock = null;
+				$value = null;
+
+				switch ($m) {
 					case 'ITEM.VALUE':
-						if ($events) {
+						if ($options['events']) {
 							$trigger = $triggers[$function['triggerid']];
-							$value = Manager::History()->getValueAt($function, $trigger['clock'], $trigger['ns']);
+							$history = Manager::History()->getValueAt($function, $trigger['clock'], $trigger['ns']);
+
+							if (is_array($history)) {
+								if (array_key_exists('clock', $history)) {
+									$clock = $history['clock'];
+								}
+								if (array_key_exists('value', $history)) {
+									$value = $history['value'];
+								}
+							}
 							break;
 						}
 						// break; is not missing here
@@ -707,14 +785,17 @@ class CMacrosResolverGeneral {
 					case 'ITEM.LASTVALUE':
 						$history = Manager::History()->getLastValues([$function], 1, ZBX_HISTORY_PERIOD);
 
-						$value = array_key_exists($function['itemid'], $history)
-							? $history[$function['itemid']][0]['value']
-							: null;
+						if (array_key_exists($function['itemid'], $history)) {
+							$clock = $history[$function['itemid']][0]['clock'];
+							$value = $history[$function['itemid']][0]['value'];
+						}
 						break;
 				}
 
-				if ($value !== null) {
-					foreach ($tokens as $token) {
+				foreach ($tokens as $token) {
+					$macro_value = UNRESOLVED_MACRO_STRING;
+
+					if ($value !== null) {
 						if (array_key_exists('function', $token)) {
 							if ($token['function'] !== 'regsub' && $token['function'] !== 'iregsub') {
 								continue;
@@ -747,9 +828,46 @@ class CMacrosResolverGeneral {
 						else {
 							$macro_value = formatHistoryValue($value, $function);
 						}
-
-						$macro_values[$function['triggerid']][$token['token']] = $macro_value;
 					}
+
+					if ($options['html']) {
+						$macro_value = str_replace(["\r\n", "\n"], [" "], $macro_value);
+						$hint_table = (new CTable())
+							->addClass('list-table')
+							->addRow([
+								new CCol($function['name_expanded']),
+								new CCol(
+									($clock !== null)
+										? zbx_date2str(DATE_TIME_FORMAT_SECONDS, $clock)
+										: UNRESOLVED_MACRO_STRING
+								),
+								new CCol($macro_value),
+								new CCol(
+									($function['value_type'] == ITEM_VALUE_TYPE_FLOAT
+											|| $function['value_type'] == ITEM_VALUE_TYPE_UINT64)
+										? new CLink(_('Graph'), (new CUrl('history.php'))
+											->setArgument('action', HISTORY_GRAPH)
+											->setArgument('itemids[]', $function['itemid'])
+											->getUrl()
+										)
+										: new CLink(_('History'), (new CUrl('history.php'))
+											->setArgument('action', HISTORY_VALUES)
+											->setArgument('itemids[]', $function['itemid'])
+											->getUrl()
+										)
+								)
+							]);
+						$macro_value = new CSpan([
+							(new CSpan())
+								->addClass('main-hint')
+								->setHint($hint_table),
+							(new CLinkAction($macro_value))
+								->addClass('hint-item')
+								->setAttribute('data-hintbox', '1')
+						]);
+					}
+
+					$macro_values[$function['triggerid']][$token['token']] = $macro_value;
 				}
 			}
 		}
