@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2018 Zabbix SIA
+** Copyright (C) 2001-2019 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -52,26 +52,35 @@ class CView {
 	private $scripts;
 
 	/**
-	 * Java code for inclusions on page.
+	 * Javascript code for inclusions on page.
 	 *
 	 * @var array
 	 */
 	private $jsIncludePost = [];
 
 	/**
-	 * Java Script files for inclusions on page, pre-processed by PHP.
+	 * Javascript files for inclusions on page, pre-processed by PHP.
 	 *
 	 * @var array
 	 */
 	private $jsIncludeFiles = [];
 
 	/**
-	 * Java Script files for inclusions on page, included as <script src="..."></script>.
+	 * Javascript files for inclusions on page, included as <script src="..."></script>.
 	 *
 	 * @var array
 	 */
 	private $jsFiles = [];
 
+	/**
+	 * Don't include jsLoader to the page.
+	 *
+	 * @static
+	 *
+	 * @var bool
+	 */
+	public static $js_loader_disabled = false;
+
 	/**
 	 * Directories where views are stored, ordered by priority include/views should be removed once fully move to MVC.
 	 *
@@ -113,7 +122,7 @@ class CView {
 	 * @throws Exception if invalid filename or file does not exist.
 	 */
 	public function assign($view) {
-		if (!preg_match("/[a-z\.]+/", $view)) {
+		if (!preg_match("/^[a-z\.]+$/", $view)) {
 			throw new Exception(_s('Invalid view name given "%s". Allowed chars: "a-z" and ".".', $view));
 		}
 
@@ -302,4 +311,24 @@ class CView {
 			return ZBX_LAYOUT_NORMAL;
 		}
 	}
+
+	/**
+	 * Update layout mode setting
+	 *
+	 * @param int $layout_mode  Possible values ZBX_LAYOUT_NORMAL|ZBX_LAYOUT_FULLSCREEN|ZBX_LAYOUT_KIOSKMODE.
+	 */
+	public static function setLayoutMode($layout_mode) {
+		CProfile::update('web.layout.mode', $layout_mode, PROFILE_TYPE_INT);
+	}
+
+	/**
+	 * Don't include jsLoader to the page.
+	 *
+	 * @return CView
+	 */
+	public function disableJsLoader() {
+		self::$js_loader_disabled = true;
+
+		return $this;
+	}
 }
