@@ -29,7 +29,7 @@ $form = (new CForm('post', $action))
 	->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
 	->addVar('screenid', getRequest('screenid'));
 
-if ($data['screen']['templateid'] != 0) {
+if (array_key_exists('templateid', $data['screen'])) {
 	$form->addVar('templateid', $data['screen']['templateid']);
 }
 
@@ -81,7 +81,7 @@ else {
 
 // append resource types to form list
 $screenResources = screen_resources();
-if ($this->data['screen']['templateid']) {
+if (array_key_exists('templateid', $data['screen'])) {
 	unset(
 		$screenResources[SCREEN_RESOURCE_DATA_OVERVIEW], $screenResources[SCREEN_RESOURCE_ACTIONS],
 		$screenResources[SCREEN_RESOURCE_EVENTS], $screenResources[SCREEN_RESOURCE_HOST_INFO],
@@ -140,7 +140,7 @@ if ($resourceType == SCREEN_RESOURCE_GRAPH) {
 					'srcfld2' => 'name',
 					'dstfrm' => $form->getName(),
 					'dstfld1' => 'resourceid'
-				], $this->data['screen']['templateid'] ? [
+				], array_key_exists('templateid', $data['screen']) ? [
 					'templated_hosts' => '1',
 					'hostid' => $data['screen']['templateid']
 				] : [
@@ -198,7 +198,7 @@ elseif ($resourceType == SCREEN_RESOURCE_LLD_GRAPH) {
 						'srcfld2' => 'name',
 						'dstfrm' => $form->getName(),
 						'dstfld1' => 'resourceid'
-					], $this->data['screen']['templateid'] ? [
+					], array_key_exists('templateid', $data['screen']) ? [
 						'templated_hosts' => '1',
 						'hostid' => $data['screen']['templateid']
 					] : [
@@ -260,9 +260,9 @@ elseif ($resourceType == SCREEN_RESOURCE_SIMPLE_GRAPH) {
 					'dstfld1' => 'resourceid',
 					'webitems' => true,
 					'numeric' => true,
-					'hostid' => $data['screen']['templateid'],
-					'real_hosts' => ($data['screen']['templateid'] == 0),
-					'with_simple_graph_items' => ($data['screen']['templateid'] == 0)
+					'hostid' => array_key_exists('templateid', $data['screen']) ? $data['screen']['templateid'] : 0,
+					'real_hosts' => !array_key_exists('templateid', $data['screen']),
+					'with_simple_graph_items' => !array_key_exists('templateid', $data['screen'])
 				]
 			]
 		]))
@@ -294,7 +294,7 @@ elseif ($resourceType == SCREEN_RESOURCE_LLD_SIMPLE_GRAPH) {
 		$caption = $item['host']['name'].NAME_DELIMITER.$item['name_expanded'];
 	}
 
-	if ($this->data['screen']['templateid']) {
+	if (array_key_exists('templateid', $data['screen'])) {
 		$selectButton = (new CButton('select', _('Select')))
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->onClick('return PopUp("popup.generic",'.
@@ -427,8 +427,8 @@ elseif ($resourceType == SCREEN_RESOURCE_PLAIN_TEXT) {
 						'srcfld1' => 'itemid',
 						'dstfrm' => $form->getName(),
 						'dstfld1' => 'resourceid',
-						'hostid' => $data['screen']['templateid'],
-						'real_hosts' => ($data['screen']['templateid'] == 0),
+						'hostid' => array_key_exists('templateid', $data['screen']) ? $data['screen']['templateid'] : 0,
+						'real_hosts' => !array_key_exists('templateid', $data['screen']),
 						'webitems' => true
 					]
 				],
@@ -738,8 +738,8 @@ elseif ($resourceType == SCREEN_RESOURCE_CLOCK) {
 						'srcfld1' => 'itemid',
 						'dstfrm' => $form->getName(),
 						'dstfld1' => 'resourceid',
-						'hostid' => $data['screen']['templateid'],
-						'real_hosts' => ($data['screen']['templateid'] == 0),
+						'hostid' => array_key_exists('templateid', $data['screen']) ? $data['screen']['templateid'] : 0,
+						'real_hosts' => !array_key_exists('templateid', $data['screen']),
 						'webitems' => true
 					]
 				]
@@ -855,7 +855,7 @@ $resourcesWithDynamic = [
 	SCREEN_RESOURCE_LLD_GRAPH,
 	SCREEN_RESOURCE_LLD_SIMPLE_GRAPH
 ];
-if ($this->data['screen']['templateid'] == 0 && in_array($resourceType, $resourcesWithDynamic)) {
+if (!array_key_exists('templateid', $this->data['screen']) && in_array($resourceType, $resourcesWithDynamic)) {
 	$screenFormList->addRow(_('Dynamic item'), (new CCheckBox('dynamic'))->setChecked($dynamic == 1));
 }
 
