@@ -18,6 +18,40 @@
  **/
 
 
+/**
+ * An object that is used to namespace objects, allows to retrieve and write objects via arbitrary path.
+ */
+window.ZABBIX = Object.create({
+
+	/**
+	 * @param {string} path  Dot separated path. Each segment is used as object key.
+	 * @param {mixed} value  Optional value to be written into path only if path held undefined before.
+	 *
+	 * @return {mixed}  Value underlaying the path is returned.
+	 */
+	namespace: function(path, value) {
+		return path.split('.').reduce(function(obj, pt, idx, src) {
+			var last = (idx + 1 == src.length);
+
+			if (typeof obj[pt] === 'undefined') {
+				obj[pt] = last ? value : {};
+			}
+
+			return obj[pt];
+		}, this);
+	},
+
+	/**
+	 * Logs user out, also, handles side effects before that.
+	 */
+	logout: function() {
+		var ls = this.namespace('instances.localStorage');
+		ls && ls.destruct();
+
+		redirect('index.php?reconnect=1', 'post', 'sid', true);
+	}
+});
+
 jQuery(function($) {
 
 	$.propHooks.disabled = {
@@ -37,7 +71,7 @@ jQuery(function($) {
 		$search.keyup(function() {
 			$search
 				.siblings('button')
-				.attr('disabled', ($.trim($search.val()) === '') ? true : null);
+				.prop('disabled', ($.trim($search.val()) === ''));
 		}).closest('form').submit(function() {
 			if ($.trim($search.val()) === '') {
 				return false;
@@ -73,7 +107,7 @@ jQuery(function($) {
 	function uncheckedHandler($checkbox) {
 		var $hidden = $checkbox.prev('input[type=hidden][name="' + $checkbox.prop('name') + '"]');
 
-		if ($checkbox.is(':checked') || $checkbox.is(':disabled')) {
+		if ($checkbox.is(':checked') || $checkbox.prop('disabled')) {
 			$hidden.remove();
 		}
 		else if (!$hidden.length) {
@@ -92,14 +126,14 @@ jQuery(function($) {
 		});
 	});
 
-	function showMenuPopup(obj, data, event) {
+	function showMenuPopup($obj, data, event) {
 		switch (data.type) {
 			case 'history':
 				data = getMenuPopupHistory(data);
 				break;
 
 			case 'host':
-				data = getMenuPopupHost(data, obj);
+				data = getMenuPopupHost(data, $obj);
 				break;
 
 			case 'map_element_submap':
@@ -119,11 +153,11 @@ jQuery(function($) {
 				break;
 
 			case 'refresh':
-				data = getMenuPopupRefresh(data, obj);
+				data = getMenuPopupRefresh(data, $obj);
 				break;
 
 			case 'trigger':
-				data = getMenuPopupTrigger(data, obj);
+				data = getMenuPopupTrigger(data, $obj);
 				break;
 
 			case 'trigger_macro':
@@ -131,11 +165,11 @@ jQuery(function($) {
 				break;
 
 			case 'dashboard':
-				data = getMenuPopupDashboard(data, obj);
+				data = getMenuPopupDashboard(data, $obj);
 				break;
 
 			case 'item':
-				data = getMenuPopupItem(data, obj);
+				data = getMenuPopupItem(data, $obj);
 				break;
 
 			case 'item_prototype':
@@ -143,49 +177,97 @@ jQuery(function($) {
 				break;
 		}
 
-		obj.menuPopup(data, event);
+		$obj.menuPopup(data, event);
+	}
+
+	/**
+	 * Create preloader elements for the menu popup.
+	 */
+	function createMenuPopupPreloader() {
+		return $('<div>', {
+			'id': 'menu-popup-preloader',
+			'class': 'preloader-container menu-popup-preloader'
+		})
+			.append($('<div>').addClass('preloader'))
+			.appendTo($('body'))
+			.on('click', function(e) {
+				e.stopPropagation();
+			})
+			.hide();
+	}
+
+	/**
+	 * Event handler for the preloader elements destroy.
+	 */
+	function menuPopupPreloaderCloseHandler(event) {
+		overlayPreloaderDestroy(event.data.id);
 	}
 
 	/**
 	 * Build menu popup for given elements.
 	 */
 	$(document).on('keydown click', '[data-menu-popup]', function(event) {
-		var obj = $(this),
-			data = obj.data('menu-popup');
+		var $obj = $(this),
+			data = $obj.data('menu-popup'),
+			position_target = event.target;
 
-		if (event.type === 'keydown') {
-			if (event.which != 13) {
-				return;
-			}
-
-			event.preventDefault();
-			event.target = this;
+		if (event.type === 'keydown' && event.which != 13) {
+			return;
+		}
+		else if (event.type === 'contextmenu' || (IE && $obj.closest('svg').length > 0)
+			|| event.originalEvent.detail !== 0) {
+			position_target = event;
 		}
 
-		var	url = new Curl('zabbix.php'),
-			ajax_data = {
-				data: data.data
-			};
+		// Manually trigger event for menuPopupPreloaderCloseHandler call for the previous preloader.
+		if ($('#menu-popup-preloader').length) {
+			$(document).trigger('click');
+		}
+
+		var	$preloader = createMenuPopupPreloader(),
+			url = new Curl('zabbix.php');
 
 		url.setArgument('action', 'menu.popup');
 		url.setArgument('type', data.type);
 
-		$.ajax({
+		var xhr = $.ajax({
 			url: url.getUrl(),
 			method: 'POST',
-			data: ajax_data,
+			data: {
+				data: data.data
+			},
 			dataType: 'json',
+			beforeSend: function() {
+				// Close other action menus and prevent focus jumping before opening a new popup.
+				$('.menu-popup-top').menuPopup('close', null, false);
+				setTimeout(function(){
+					$preloader
+						.fadeIn(200)
+						.position({
+							of: position_target,
+							my: 'left top',
+							at: 'left bottom'
+						});
+				}, 500);
+			},
 			success: function(resp) {
-				showMenuPopup(obj, resp.data, event);
+				overlayPreloaderDestroy($preloader.prop('id'));
+				showMenuPopup($obj, resp.data, event);
 			},
 			error: function() {
 			}
 		});
 
+		addToOverlaysStack($preloader.prop('id'), event.target, 'preloader', xhr);
+
+		$(document)
+			.off('click', menuPopupPreloaderCloseHandler)
+			.on('click', {id: $preloader.prop('id')}, menuPopupPreloaderCloseHandler);
+
 		return false;
 	});
 
-	/*
+	/**
 	 * add.popup event
 	 *
 	 * Call multiselect method 'addData' if parent was multiselect, execute addPopupValues function
@@ -199,6 +281,7 @@ jQuery(function($) {
 	$(document).on('add.popup', function(e, data) {
 		// multiselect check
 		if ($('#' + data.parentId).hasClass('multiselect')) {
+			var items = [];
 			for (var i = 0; i < data.values.length; i++) {
 				if (typeof data.values[i].id !== 'undefined') {
 					var item = {
@@ -209,34 +292,11 @@ jQuery(function($) {
 					if (typeof data.values[i].prefix !== 'undefined') {
 						item.prefix = data.values[i].prefix;
 					}
-
-					$('#' + data.parentId).multiSelect('addData', item);
+					items.push(item);
 				}
 			}
-		}
-		else if ($('[name="' + data.parentId + '"]').hasClass('patternselect')) {
-			/**
-			 * Pattern select allows to enter multiple comma or newline separated values in same editable field. Values
-			 * passed to add.popup should be appended at the end of existing value string.
-			 *
-			 * values_arr is used to catch duplicates.
-			 * values_str is used to store user's original syntax.
-			 */
-			var values_str = $('[name="' + data.parentId + '"]').val(),
-				values_arr = values_str.split(/[,|\n]+/).map(function(str) {return str.trim()});
 
-			data.values.forEach(function(val) {
-				if (values_arr.indexOf(val[data.object]) == -1) {
-					if (values_str !== '') {
-						values_str += ', ';
-					}
-					values_str += val[data.object];
-				}
-			});
-
-			$('[name="' + data.parentId + '"]')
-				.val(values_str)
-				.trigger('change');
+			$('#' + data.parentId).multiSelect('addData', items);
 		}
 		else if (!$('[name="' + data.parentId + '"]').hasClass('simple-textbox')
 				&& typeof addPopupValues !== 'undefined') {
